@@ -1067,7 +1067,7 @@ int msLoadMapContextURL(mapObj *map, char *urlfilename, int unique_layer_names)
   }
 
   pszTmpFile = msTmpFile(map, map->mappath, NULL, "context.xml");
-  if (msHTTPGetFile(urlfilename, pszTmpFile, &status,-1, 0, 0) ==  MS_SUCCESS) {
+  if (msHTTPGetFile(urlfilename, pszTmpFile, &status,-1, 0, 0, 0) ==  MS_SUCCESS) {
     return msLoadMapContext(map, pszTmpFile, unique_layer_names);
   } else {
     msSetError(MS_MAPCONTEXTERR,
