@@ -1202,7 +1202,7 @@ msPostGISRetrievePK(layerObj *layer)
     tmp2 = (char*)msSmallMalloc(size2);
     strlcpy(tmp2, tmp1, size2);
     strlcat(tmp2, sql, size2);
-    msSetError(MS_QUERYERR, tmp2, "msPostGISRetrievePK()");
+    msSetError(MS_QUERYERR, "%s", "msPostGISRetrievePK()", tmp2);
     free(tmp2);
     free(sql);
     return MS_FAILURE;
@@ -1729,7 +1729,7 @@ char *msPostGISBuildSQLSRID(layerObj *layer)
     char *strSRIDTemplate = "find_srid('','%s','%s')";
     char *pos = strstr(layerinfo->fromsource, " ");
     if( layer->debug > 1 ) {
-      msDebug("msPostGISBuildSQLSRID: Building find_srid line.\n", strSRID);
+      msDebug("msPostGISBuildSQLSRID: Building find_srid line.\n");
     }
 
     if ( ! pos ) {
@@ -2183,14 +2183,14 @@ int msPostGISReadShape(layerObj *layer, shapeObj *shape)
       uid = 0;
     }
     if( layer->debug > 4 ) {
-      msDebug("msPostGISReadShape: Setting shape->index = %d\n", uid);
-      msDebug("msPostGISReadShape: Setting shape->resultindex = %d\n", layerinfo->rownum);
+      msDebug("msPostGISReadShape: Setting shape->index = %ld\n", uid);
+      msDebug("msPostGISReadShape: Setting shape->resultindex = %ld\n", layerinfo->rownum);
     }
     shape->index = uid;
     shape->resultindex = layerinfo->rownum;
 
     if( layer->debug > 2 ) {
-      msDebug("msPostGISReadShape: [index] %d\n",  shape->index);
+      msDebug("msPostGISReadShape: [index] %ld\n",  shape->index);
     }
 
     shape->numvalues = layer->numitems;
@@ -2296,8 +2296,7 @@ int msPostGISLayerOpen(layerObj *layer)
         }
       }
 
-      msDebug("Database connection failed (%s) with connect string '%s'\nIs the database running? Is it allowing connections? Does the specified user exist? Is the password valid? Is the database on the standard port?.\n", PQerrorMessage(layerinfo->pgconn), maskeddata);
-      msSetError(MS_QUERYERR, "Database connection failed.\nIs the database running? Is it allowing connections? Does the specified user exist? Is the password valid? Is the database on the standard port?", "msPostGISLayerOpen()");
+      msSetError(MS_QUERYERR, "Database connection failed (%s) with connect string '%s'\nIs the database running? Is it allowing connections? Does the specified user exist? Is the password valid? Is the database on the standard port?", "msPostGISLayerOpen()", PQerrorMessage(layerinfo->pgconn), maskeddata);
 
       free(maskeddata);
       free(layerinfo);
@@ -2319,6 +2318,7 @@ int msPostGISLayerOpen(layerObj *layer)
         msSetError(MS_QUERYERR, "PostgreSQL database connection gone bad (%s)", "msPostGISLayerOpen()", PQerrorMessage(layerinfo->pgconn));
         return MS_FAILURE;
       }
+
     }
   }
 
@@ -2532,7 +2532,7 @@ int msPostGISLayerWhichShapes(layerObj *layer, rectObj rect, int isQuery)
   /* Something went wrong. */
   if (!pgresult || PQresultStatus(pgresult) != PGRES_TUPLES_OK) {
     if ( layer->debug ) {
-      msDebug("Error (%s) executing query: %s", "msPostGISLayerWhichShapes()\n", PQerrorMessage(layerinfo->pgconn), strSQL);
+      msDebug("msPostGISLayerWhichShapes(): Error (%s) executing query: %s\n", PQerrorMessage(layerinfo->pgconn), strSQL);
     }
     msSetError(MS_QUERYERR, "Error executing query: %s ", "msPostGISLayerWhichShapes()", PQerrorMessage(layerinfo->pgconn));
     free(strSQL);
@@ -2713,7 +2713,7 @@ int msPostGISLayerGetShape(layerObj *layer, shapeObj *shape, resultObj *record)
     /* Something went wrong. */
     if ( (!pgresult) || (PQresultStatus(pgresult) != PGRES_TUPLES_OK) ) {
       if ( layer->debug ) {
-        msDebug("Error (%s) executing SQL: %s", "msPostGISLayerGetShape()\n", PQerrorMessage(layerinfo->pgconn), strSQL );
+        msDebug("msPostGISLayerGetShape(): Error (%s) executing SQL: %s\n", PQerrorMessage(layerinfo->pgconn), strSQL );
       }
       msSetError(MS_QUERYERR, "Error executing SQL: %s", "msPostGISLayerGetShape()", PQerrorMessage(layerinfo->pgconn));
 
@@ -2939,7 +2939,7 @@ int msPostGISLayerGetItems(layerObj *layer)
 
   if ( (!pgresult) || (PQresultStatus(pgresult) != PGRES_TUPLES_OK) ) {
     if ( layer->debug ) {
-      msDebug("Error (%s) executing SQL: %s", "msPostGISLayerGetItems()\n", PQerrorMessage(layerinfo->pgconn), sql);
+      msDebug("msPostGISLayerGetItems(): Error (%s) executing SQL: %s\n", PQerrorMessage(layerinfo->pgconn), sql);
     }
     msSetError(MS_QUERYERR, "Error executing SQL: %s", "msPostGISLayerGetItems()", PQerrorMessage(layerinfo->pgconn));
     if (pgresult) {
@@ -3212,11 +3212,6 @@ int msPostGISLayerSetTimeFilter(layerObj *lp, const char *timestring, const char
   if (!lp || !timestring || !timefield)
     return MS_FALSE;
 
-  if( strchr(timestring,'\'') || strchr(timestring, '\\') ) {
-     msSetError(MS_MISCERR, "Invalid time filter.", "msPostGISLayerSetTimeFilter()");
-     return MS_FALSE;
-  }
-
   /* discrete time */
   if (strstr(timestring, ",") == NULL &&
       strstr(timestring, "/") == NULL) { /* discrete time */
