@@ -46,6 +46,12 @@
 #include <process.h>
 #endif
 
+#ifdef USE_RSVG
+#include <glib-object.h>
+#endif
+#ifdef USE_GEOS
+#include <geos_c.h>
+#endif
 
 
 extern char *msyystring_buffer;
@@ -199,7 +205,7 @@ static void bindLabel(layerObj *layer, shapeObj *shape, labelObj *label, int dra
   /* check the label styleObj's (TODO: do we need to use querymapMode here? */
   for(i=0; i<label->numstyles; i++) {
     /* force MS_DRAWMODE_FEATURES for label styles */
-    bindStyle(layer, shape, label->styles[i], drawmode|MS_DRAWMODE_FEATURES); 
+    bindStyle(layer, shape, label->styles[i], drawmode|MS_DRAWMODE_FEATURES);
   }
 
   if(label->numbindings > 0) {
@@ -466,6 +472,25 @@ int msEvalExpression(layerObj *layer, shapeObj *shape, expressionObj *expression
         if(strcmp(expression->string, shape->values[itemindex]) == 0) return MS_TRUE; /* got a match */
       }
       break;
+    case(MS_LIST):
+      if(itemindex == -1) {
+        msSetError(MS_MISCERR, "Cannot evaluate expression, no item index defined.", "msEvalExpression()");
+        return MS_FALSE;
+      }
+      if(itemindex >= layer->numitems || itemindex >= shape->numvalues) {
+        msSetError(MS_MISCERR, "Invalid item index.", "msEvalExpression()");
+        return MS_FALSE;
+      }
+      {
+        char *start,*end;
+        start = expression->string;
+        while((end = strchr(start,',')) != NULL) {
+          if(!strncmp(start,shape->values[itemindex],end-start)) return MS_TRUE;
+          start = end+1;
+        }
+        if(!strcmp(start,shape->values[itemindex])) return MS_TRUE;
+      }
+      break;
     case(MS_EXPRESSION): {
       int status;
       parseObj p;
@@ -1610,18 +1635,23 @@ imageObj *msImageCreate(int width, int height, outputFormatObj *format,
 void  msTransformPoint(pointObj *point, rectObj *extent, double cellsize,
                        imageObj *image)
 {
+  double invcellsize;
   /*We should probabaly have a function defined at all the renders*/
-  if (image != NULL && MS_RENDERER_PLUGIN(image->format) &&
-      image->format->renderer == MS_RENDER_WITH_KML)
-    return;
-
-  point->x = MS_MAP2IMAGE_X(point->x, extent->minx, cellsize);
-  point->y = MS_MAP2IMAGE_Y(point->y, extent->maxy, cellsize);
+  if (image != NULL && MS_RENDERER_PLUGIN(image->format)) {
+    if(image->format->renderer == MS_RENDER_WITH_KML) {
+      return;
+    } else if(image->format->renderer == MS_RENDER_WITH_GD) {
+      point->x = MS_MAP2IMAGE_X(point->x, extent->minx, cellsize);
+      point->y = MS_MAP2IMAGE_Y(point->y, extent->maxy, cellsize);
+      return;
+    }
+  }
+  invcellsize = 1.0/cellsize;
+  point->x = MS_MAP2IMAGE_X_IC_DBL(point->x, extent->minx, invcellsize);
+  point->y = MS_MAP2IMAGE_Y_IC_DBL(point->y, extent->maxy, invcellsize);
 }
 
 
-
-
 /*
 ** Helper functions supplied as part of bug #2868 solution. Consider moving these to
 ** mapprimitive.c for more general use.
@@ -1717,18 +1747,119 @@ static double point_cross(const pointObj a, const pointObj b)
   return a.x*b.y-a.y*b.x;
 }
 
-/*
-** For offset corner point calculation 1/sin() is used
-** to avoid 1/0 division (and long spikes) we define a
-** limit for sin().
-*/
+shapeObj *msOffsetCurve(shapeObj *p, double offset)
+{
+  shapeObj *ret;
+  int i, j, first,idx,ok=0;
+#if defined USE_GEOS && (GEOS_VERSION_MAJOR > 3 || (GEOS_VERSION_MAJOR == 3 && GEOS_VERSION_MINOR >= 3))
+  ret = msGEOSOffsetCurve(p,offset);
+  /* GEOS curve offsetting can fail sometimes, we continue with our own implementation
+   if that is the case.*/
+  if(ret)
+    return ret;
+#endif
+  /*
+  ** For offset corner point calculation 1/sin() is used
+  ** to avoid 1/0 division (and long spikes) we define a
+  ** limit for sin().
+  */
 #define CURVE_SIN_LIMIT 0.3
+  ret = (shapeObj*)msSmallMalloc(sizeof(shapeObj));
+  msInitShape(ret);
+  ret->numlines = p->numlines;
+  ret->line=(lineObj*)msSmallMalloc(sizeof(lineObj)*ret->numlines);
+  for(i=0; i<ret->numlines; i++) {
+    ret->line[i].numpoints=p->line[i].numpoints;
+    ret->line[i].point=(pointObj*)msSmallMalloc(sizeof(pointObj)*ret->line[i].numpoints);
+  }
+  for (i = 0; i < p->numlines; i++) {
+    pointObj old_pt, old_diffdir, old_offdir;
+    if(p->line[i].numpoints<2) {
+      ret->line[i].numpoints = 0;
+      continue; /* skip degenerate points */
+    }
+    ok = 1;
+    /* initialize old_offdir and old_diffdir, as gcc isn't smart enough to see that it
+     * is not an error to do so, and prints a warning */
+    old_offdir.x=old_offdir.y=old_diffdir.x=old_diffdir.y = 0;
+
+    idx=0;
+    first = 1;
+
+    /* saved metrics of the last processed point */
+    if (p->line[i].numpoints>0)
+      old_pt=p->line[i].point[0];
+    for(j=1; j<p->line[i].numpoints; j++) {
+      const pointObj pt = p->line[i].point[j]; /* place of the point */
+      const pointObj diffdir = point_norm(point_diff(pt,old_pt)); /* direction of the line */
+      const pointObj offdir = point_rotz90(diffdir); /* direction where the distance between the line and the offset is measured */
+      pointObj offpt; /* this will be the corner point of the offset line */
+
+      /* offset line points computation */
+      if(first == 1) { /* first point */
+        first = 0;
+        offpt = point_sum(old_pt,point_mul(offdir,offset));
+      } else { /* middle points */
+        /* curve is the angle of the last and the current line's direction (supplementary angle of the shape's inner angle) */
+        double sin_curve = point_cross(diffdir,old_diffdir);
+        double cos_curve = point_cross(old_offdir,diffdir);
+        if ((-1.0)*CURVE_SIN_LIMIT < sin_curve && sin_curve < CURVE_SIN_LIMIT) {
+          /* do not calculate 1/sin, instead use a corner point approximation: average of the last and current offset direction and length */
+
+          /*
+          ** TODO: fair for obtuse inner angles, however, positive and negative
+          ** acute inner angles would need special handling - similar to LINECAP
+          ** to avoid drawing of long spikes
+          */
+          offpt = point_sum(old_pt, point_mul(point_sum(offdir, old_offdir),0.5*offset));
+        } else {
+          double base_shift = -1.0*(1.0+cos_curve)/sin_curve;
+          offpt = point_sum(old_pt, point_mul(point_sum(point_mul(diffdir,base_shift),offdir), offset));
+        }
+      }
+      ret->line[i].point[idx]=offpt;
+      idx++;
+      old_pt=pt;
+      old_diffdir=diffdir;
+      old_offdir=offdir;
+    }
+
+    /* last point */
+    if(first == 0) {
+      pointObj offpt=point_sum(old_pt,point_mul(old_offdir,offset));
+      ret->line[i].point[idx]=offpt;
+      idx++;
+    }
+
+    if(idx != p->line[i].numpoints) {
+      /* printf("shouldn't happen :(\n"); */
+      ret->line[i].numpoints=idx;
+      ret->line=msSmallRealloc(ret->line,ret->line[i].numpoints*sizeof(pointObj));
+    }
+  }
+  if(!ok) ret->numlines = 0; /* all lines where degenerate */
+  return ret;
+}
 
 shapeObj *msOffsetPolyline(shapeObj *p, double offsetx, double offsety)
 {
-  int i, j, first,idx;
+  int i, j;
+  shapeObj *ret;
+  if(offsety == MS_STYLE_SINGLE_SIDED_OFFSET) { /* complex calculations */
+    return msOffsetCurve(p,offsetx);
+  } else if(offsety == MS_STYLE_DOUBLE_SIDED_OFFSET) {
+    shapeObj *tmp1;
+    ret = msOffsetCurve(p,offsetx/2.0);
+    tmp1 = msOffsetCurve(p, -offsetx/2.0);
+    for(i=0;i<tmp1->numlines; i++) {
+      msAddLineDirectly(ret,tmp1->line + i);
+    }
+    msFreeShape(tmp1);
+    free(tmp1);
+    return ret;
+  }
 
-  shapeObj *ret = (shapeObj*)msSmallMalloc(sizeof(shapeObj));
+  ret = (shapeObj*)msSmallMalloc(sizeof(shapeObj));
   msInitShape(ret);
   ret->numlines = p->numlines;
   ret->line=(lineObj*)msSmallMalloc(sizeof(lineObj)*ret->numlines);
@@ -1737,80 +1868,10 @@ shapeObj *msOffsetPolyline(shapeObj *p, double offsetx, double offsety)
     ret->line[i].point=(pointObj*)msSmallMalloc(sizeof(pointObj)*ret->line[i].numpoints);
   }
 
-  if(offsety == -99) { /* complex calculations */
-    int ok = 0;
-    for (i = 0; i < p->numlines; i++) {
-      pointObj old_pt, old_diffdir, old_offdir;
-      if(p->line[i].numpoints<2) {
-        ret->line[i].numpoints = 0;
-        continue; /* skip degenerate lines */
-      }
-      ok =1;
-      /* initialize old_offdir and old_diffdir, as gcc isn't smart enough to see that it
-       * is not an error to do so, and prints a warning */
-      old_offdir.x=old_offdir.y=old_diffdir.x=old_diffdir.y = 0;
-
-      idx=0;
-      first = 1;
-
-      /* saved metrics of the last processed point */
-      if (p->line[i].numpoints>0)
-        old_pt=p->line[i].point[0];
-      for(j=1; j<p->line[i].numpoints; j++) {
-        const pointObj pt = p->line[i].point[j]; /* place of the point */
-        const pointObj diffdir = point_norm(point_diff(pt,old_pt)); /* direction of the line */
-        const pointObj offdir = point_rotz90(diffdir); /* direction where the distance between the line and the offset is measured */
-        pointObj offpt; /* this will be the corner point of the offset line */
-
-        /* offset line points computation */
-        if(first == 1) { /* first point */
-          first = 0;
-          offpt = point_sum(old_pt,point_mul(offdir,offsetx));
-        } else { /* middle points */
-          /* curve is the angle of the last and the current line's direction (supplementary angle of the shape's inner angle) */
-          double sin_curve = point_cross(diffdir,old_diffdir);
-          double cos_curve = point_cross(old_offdir,diffdir);
-          if ((-1.0)*CURVE_SIN_LIMIT < sin_curve && sin_curve < CURVE_SIN_LIMIT) {
-            /* do not calculate 1/sin, instead use a corner point approximation: average of the last and current offset direction and length */
-
-            /*
-            ** TODO: fair for obtuse inner angles, however, positive and negative
-            ** acute inner angles would need special handling - similar to LINECAP
-            ** to avoid drawing of long spikes
-            */
-            offpt = point_sum(old_pt, point_mul(point_sum(offdir, old_offdir),0.5*offsetx));
-          } else {
-            double base_shift = -1.0*(1.0+cos_curve)/sin_curve;
-            offpt = point_sum(old_pt, point_mul(point_sum(point_mul(diffdir,base_shift),offdir), offsetx));
-          }
-        }
-        ret->line[i].point[idx]=offpt;
-        idx++;
-        old_pt=pt;
-        old_diffdir=diffdir;
-        old_offdir=offdir;
-      }
-
-      /* last point */
-      if(first == 0) {
-        pointObj offpt=point_sum(old_pt,point_mul(old_offdir,offsetx));
-        ret->line[i].point[idx]=offpt;
-        idx++;
-      }
-
-      if(idx != p->line[i].numpoints) {
-        /* printf("shouldn't happen :(\n"); */
-        ret->line[i].numpoints=idx;
-        ret->line=msSmallRealloc(ret->line,ret->line[i].numpoints*sizeof(pointObj));
-      }
-    }
-    if(!ok) ret->numlines = 0; /* all lines where degenerate */
-  } else { /* normal offset (eg. drop shadow) */
-    for (i = 0; i < p->numlines; i++) {
-      for(j=0; j<p->line[i].numpoints; j++) {
-        ret->line[i].point[j].x=p->line[i].point[j].x+offsetx;
-        ret->line[i].point[j].y=p->line[i].point[j].y+offsety;
-      }
+  for (i = 0; i < p->numlines; i++) {
+    for(j=0; j<p->line[i].numpoints; j++) {
+      ret->line[i].point[j].x=p->line[i].point[j].x+offsetx;
+      ret->line[i].point[j].y=p->line[i].point[j].y+offsety;
     }
   }
 
@@ -1845,11 +1906,22 @@ int msSetup()
   msGEOSSetup();
 #endif
 
+#ifdef USE_RSVG
+#if !GLIB_CHECK_VERSION(2, 35, 0)
+  g_type_init();
+#endif
+#endif
+
   return MS_SUCCESS;
 }
 
 /* This is intended to be a function to cleanup anything that "hangs around"
    when all maps are destroyed, like Registered GDAL drivers, and so forth. */
+#ifndef NDEBUG
+#if defined(USE_LIBXML2)
+#include "maplibxml2.h"
+#endif
+#endif
 void msCleanup(int signal)
 {
   msForceTmpFileBase( NULL );
@@ -2058,7 +2130,7 @@ int msCheckParentPointer(void* p, char *objname)
     } else {
       msg="A required parent object is null";
     }
-    msSetError(MS_NULLPARENTERR, msg, "");
+    msSetError(MS_NULLPARENTERR, "%s", "", msg);
     return MS_FAILURE;
   }
   return MS_SUCCESS;
@@ -2374,3 +2446,61 @@ int msMapSetLayerProjections(mapObj* map)
   msFree(mapProjStr);
   return(MS_SUCCESS);
 }
+
+/* Generalize a shape based of the tolerance.
+   Ref: http://trac.osgeo.org/gdal/ticket/966 */
+shapeObj* msGeneralize(shapeObj *shape, double tolerance)
+{
+  shapeObj *newShape;
+  lineObj newLine = {0,NULL};
+  double sqTolerance = tolerance*tolerance;
+  
+  double dX0, dY0, dX1, dY1, dX, dY, dSqDist;
+  int i;
+
+  newShape = (shapeObj*)msSmallMalloc(sizeof(shapeObj));
+  msInitShape(newShape);
+  msCopyShape(shape, newShape);
+
+  if (shape->numlines<1)
+    return newShape;
+  
+  /* Clean shape */
+  for (i=0; i < newShape->numlines; i++)
+    free(newShape->line[i].point);
+  newShape->numlines = 0;
+  if (newShape->line) free(newShape->line);
+    
+  msAddLine(newShape, &newLine);
+  
+  if (shape->line[0].numpoints>0) {
+    msAddPointToLine(&newShape->line[0],
+                     &shape->line[0].point[0]);              
+    dX0 = shape->line[0].point[0].x;
+    dY0 = shape->line[0].point[0].y;    
+  }
+  
+  for(i=1; i<shape->line[0].numpoints; i++)
+  {
+      dX1 = shape->line[0].point[i].x;
+      dY1 = shape->line[0].point[i].y;
+     
+      dX = dX1-dX0;
+      dY = dY1-dY0;
+      dSqDist = dX*dX + dY*dY;
+      if (i == shape->line[0].numpoints-1 || dSqDist >= sqTolerance)
+      {
+          pointObj p;
+          p.x = dX1;
+          p.y = dY1;
+          
+          /* Keep this point (always keep the last point) */
+          msAddPointToLine(&newShape->line[0],
+                           &p);          
+          dX0 = dX1;
+          dY0 = dY1;
+        }
+    }
+   
+  return newShape;
+}
